@@ -110,19 +110,38 @@ queue_greeter_reset (GdmFactorySlave *slave)
 }
 
 static void
-on_greeter_start (GdmGreeterSession *greeter,
-                  GdmFactorySlave   *slave)
+on_greeter_session_start (GdmGreeterSession *greeter,
+                          GdmFactorySlave   *slave)
 {
         g_debug ("GdmFactorySlave: Greeter started");
 }
 
 static void
-on_greeter_stop (GdmGreeterSession *greeter,
-                 GdmFactorySlave   *slave)
+on_greeter_session_stop (GdmGreeterSession *greeter,
+                         GdmFactorySlave   *slave)
 {
         g_debug ("GdmFactorySlave: Greeter stopped");
 }
 
+static void
+on_greeter_session_exited (GdmGreeterSession    *greeter,
+                           int                   code,
+                           GdmFactorySlave      *slave)
+{
+        g_debug ("GdmSimpleSlave: Greeter exited: %d", code);
+        gdm_slave_stopped (GDM_SLAVE (slave));
+}
+
+static void
+on_greeter_session_died (GdmGreeterSession    *greeter,
+                         int                   signal,
+                         GdmFactorySlave      *slave)
+{
+        g_debug ("GdmSimpleSlave: Greeter died: %d", signal);
+        gdm_slave_stopped (GDM_SLAVE (slave));
+}
+
+
 static void
 on_session_info (GdmSession      *session,
                  const char      *text,
@@ -506,17 +525,25 @@ run_greeter (GdmFactorySlave *slave)
                                                         display_is_local);
         g_signal_connect (slave->priv->greeter,
                           "started",
-                          G_CALLBACK (on_greeter_start),
+                          G_CALLBACK (on_greeter_session_start),
                           slave);
         g_signal_connect (slave->priv->greeter,
                           "stopped",
-                          G_CALLBACK (on_greeter_stop),
+                          G_CALLBACK (on_greeter_session_stop),
+                          slave);
+        g_signal_connect (slave->priv->greeter,
+                          "exited",
+                          G_CALLBACK (on_greeter_session_exited),
+                          slave);
+        g_signal_connect (slave->priv->greeter,
+                          "died",
+                          G_CALLBACK (on_greeter_session_died),
                           slave);
         g_object_set (slave->priv->greeter,
                       "x11-authority-file", auth_file,
                       NULL);
-        gdm_greeter_session_set_server_address (slave->priv->greeter, address);
-        gdm_greeter_session_start (slave->priv->greeter);
+        gdm_welcome_session_set_server_address (GDM_WELCOME_SESSION (slave->priv->greeter), address);
+        gdm_welcome_session_start (GDM_WELCOME_SESSION (slave->priv->greeter));
 
         g_free (address);
 
@@ -547,20 +574,43 @@ idle_connect_to_display (GdmFactorySlave *slave)
                         g_warning ("Unable to connect to display after %d tries - bailing out", slave->priv->connection_attempts);
                         exit (1);
                 }
+                return TRUE;
         }
 
         return FALSE;
 }
 
 static void
-server_ready_cb (GdmServer *server,
-                 GdmFactorySlave  *slave)
+on_server_ready (GdmServer       *server,
+                 GdmFactorySlave *slave)
 {
         g_debug ("GdmFactorySlave: Server ready");
 
         g_timeout_add (500, (GSourceFunc)idle_connect_to_display, slave);
 }
 
+static void
+on_server_exited (GdmServer       *server,
+                  int              exit_code,
+                  GdmFactorySlave *slave)
+{
+        g_debug ("GdmFactorySlave: server exited with code %d\n", exit_code);
+
+        gdm_slave_stopped (GDM_SLAVE (slave));
+}
+
+static void
+on_server_died (GdmServer       *server,
+                int              signal_number,
+                GdmFactorySlave *slave)
+{
+        g_debug ("GdmFactorySlave: server died with signal %d, (%s)",
+                 signal_number,
+                 g_strsignal (signal_number));
+
+        gdm_slave_stopped (GDM_SLAVE (slave));
+}
+
 static gboolean
 gdm_factory_slave_run (GdmFactorySlave *slave)
 {
@@ -580,10 +630,17 @@ gdm_factory_slave_run (GdmFactorySlave *slave)
                 gboolean res;
 
                 slave->priv->server = gdm_server_new (display_name, auth_file);
-
+                g_signal_connect (slave->priv->server,
+                                  "exited",
+                                  G_CALLBACK (on_server_exited),
+                                  slave);
+                g_signal_connect (slave->priv->server,
+                                  "died",
+                                  G_CALLBACK (on_server_died),
+                                  slave);
                 g_signal_connect (slave->priv->server,
                                   "ready",
-                                  G_CALLBACK (server_ready_cb),
+                                  G_CALLBACK (on_server_ready),
                                   slave);
 
                 res = gdm_server_start (slave->priv->server);
@@ -728,7 +785,7 @@ gdm_factory_slave_stop (GdmSlave *slave)
         }
 
         if (GDM_FACTORY_SLAVE (slave)->priv->greeter != NULL) {
-                gdm_greeter_session_stop (GDM_FACTORY_SLAVE (slave)->priv->greeter);
+                gdm_welcome_session_stop (GDM_WELCOME_SESSION (GDM_FACTORY_SLAVE (slave)->priv->greeter));
                 g_object_unref (GDM_FACTORY_SLAVE (slave)->priv->greeter);
                 GDM_FACTORY_SLAVE (slave)->priv->greeter = NULL;
         }
