@@ -322,18 +322,41 @@ idle_connect_to_display (GdmProductSlave *slave)
                         g_warning ("Unable to connect to display after %d tries - bailing out", slave->priv->connection_attempts);
                         exit (1);
                 }
+                return TRUE;
         }
 
         return FALSE;
 }
 
 static void
-server_ready_cb (GdmServer       *server,
+on_server_ready (GdmServer       *server,
                  GdmProductSlave *slave)
 {
         g_timeout_add (500, (GSourceFunc)idle_connect_to_display, slave);
 }
 
+static void
+on_server_exited (GdmServer       *server,
+                  int              exit_code,
+                  GdmProductSlave *slave)
+{
+        g_debug ("GdmProductSlave: server exited with code %d\n", exit_code);
+
+        gdm_slave_stopped (GDM_SLAVE (slave));
+}
+
+static void
+on_server_died (GdmServer       *server,
+                int              signal_number,
+                GdmProductSlave *slave)
+{
+        g_debug ("GdmProductSlave: server died with signal %d, (%s)",
+                 signal_number,
+                 g_strsignal (signal_number));
+
+        gdm_slave_stopped (GDM_SLAVE (slave));
+}
+
 static gboolean
 gdm_product_slave_create_server (GdmProductSlave *slave)
 {
@@ -353,10 +376,17 @@ gdm_product_slave_create_server (GdmProductSlave *slave)
                 gboolean res;
 
                 slave->priv->server = gdm_server_new (display_name, auth_file);
-
+                g_signal_connect (slave->priv->server,
+                                  "exited",
+                                  G_CALLBACK (on_server_exited),
+                                  slave);
+                g_signal_connect (slave->priv->server,
+                                  "died",
+                                  G_CALLBACK (on_server_died),
+                                  slave);
                 g_signal_connect (slave->priv->server,
                                   "ready",
-                                  G_CALLBACK (server_ready_cb),
+                                  G_CALLBACK (on_server_ready),
                                   slave);
 
                 res = gdm_server_start (slave->priv->server);
@@ -674,6 +704,7 @@ create_new_session (GdmProductSlave *slave)
         char          *display_name;
         char          *display_hostname;
         char          *display_device;
+        char          *display_x11_authority_file;
 
         g_debug ("GdmProductSlave: Creating new session");
 
@@ -681,6 +712,7 @@ create_new_session (GdmProductSlave *slave)
                       "display-name", &display_name,
                       "display-hostname", &display_hostname,
                       "display-is-local", &display_is_local,
+                      "display-x11-authority-file", &display_x11_authority_file,
                       NULL);
 
         /* FIXME: we don't yet have a display device! */
@@ -689,6 +721,7 @@ create_new_session (GdmProductSlave *slave)
         slave->priv->session = gdm_session_direct_new (display_name,
                                                        display_hostname,
                                                        display_device,
+                                                       display_x11_authority_file,
                                                        display_is_local);
         g_free (display_name);
         g_free (display_hostname);
