@@ -43,7 +43,7 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <glib-object.h>
-#define DBUS_API_SUBJECT_TO_CHANGE
+
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
@@ -82,6 +82,7 @@ struct _GdmSessionDirectPrivate
         char                *display_name;
         char                *display_hostname;
         char                *display_device;
+        char                *display_x11_authority_file;
         gboolean             display_is_local;
 
         DBusServer          *server;
@@ -91,10 +92,12 @@ struct _GdmSessionDirectPrivate
 
 enum {
         PROP_0,
+        PROP_SERVICE_NAME,
         PROP_DISPLAY_NAME,
         PROP_DISPLAY_HOSTNAME,
         PROP_DISPLAY_IS_LOCAL,
         PROP_DISPLAY_DEVICE,
+        PROP_DISPLAY_X11_AUTHORITY_FILE,
         PROP_USER_X11_AUTHORITY_FILE,
 };
 
@@ -447,6 +450,18 @@ gdm_session_direct_handle_accreditation_failed (GdmSessionDirect *session,
         return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+static void
+gdm_session_direct_select_user (GdmSession *session,
+                                const char *text)
+{
+        GdmSessionDirect *impl = GDM_SESSION_DIRECT (session);
+
+        g_debug ("GdmSessionDirect: Setting user: '%s'", text);
+
+        g_free (impl->priv->selected_user);
+        impl->priv->selected_user = g_strdup (text);
+}
+
 static DBusHandlerResult
 gdm_session_direct_handle_username_changed (GdmSessionDirect *session,
                                             DBusConnection   *connection,
@@ -471,8 +486,7 @@ gdm_session_direct_handle_username_changed (GdmSessionDirect *session,
                  session->priv->selected_user != NULL ? session->priv->selected_user : "<unset>",
                  (strlen (text)) ? text : "<unset>");
 
-        g_free (session->priv->selected_user);
-        session->priv->selected_user = (strlen (text) > 0) ? g_strdup (text) : NULL;
+        gdm_session_direct_select_user (GDM_SESSION (session), (strlen (text) > 0) ? g_strdup (text) : NULL);
 
         _gdm_session_selected_user_changed (GDM_SESSION (session), session->priv->selected_user);
 
@@ -898,12 +912,14 @@ do_introspect (DBusConnection *connection,
                                "      <arg name=\"x11_display_name\" type=\"s\"/>\n"
                                "      <arg name=\"display_device\" type=\"s\"/>\n"
                                "      <arg name=\"hostname\" type=\"s\"/>\n"
+                               "      <arg name=\"x11_authority_file\" type=\"s\"/>\n"
                                "    </signal>\n"
                                "    <signal name=\"SetupForUser\">\n"
                                "      <arg name=\"service_name\" type=\"s\"/>\n"
                                "      <arg name=\"x11_display_name\" type=\"s\"/>\n"
                                "      <arg name=\"display_device\" type=\"s\"/>\n"
                                "      <arg name=\"hostname\" type=\"s\"/>\n"
+                               "      <arg name=\"x11_authority_file\" type=\"s\"/>\n"
                                "      <arg name=\"username\" type=\"s\"/>\n"
                                "    </signal>\n"
                                "    <signal name=\"Authenticate\">\n"
@@ -1281,6 +1297,7 @@ send_setup (GdmSessionDirect *session)
         const char     *display_name;
         const char     *display_device;
         const char     *display_hostname;
+        const char     *display_x11_authority_file;
 
         if (session->priv->display_name != NULL) {
                 display_name = session->priv->display_name;
@@ -1297,6 +1314,11 @@ send_setup (GdmSessionDirect *session)
         } else {
                 display_device = "";
         }
+        if (session->priv->display_x11_authority_file != NULL) {
+                display_x11_authority_file = session->priv->display_x11_authority_file;
+        } else {
+                display_x11_authority_file = "";
+        }
 
         g_debug ("GdmSessionDirect: Beginning setup");
 
@@ -1309,6 +1331,7 @@ send_setup (GdmSessionDirect *session)
         dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &display_name);
         dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &display_device);
         dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &display_hostname);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &display_x11_authority_file);
 
         if (! send_dbus_message (session->priv->worker_connection, message)) {
                 g_debug ("GdmSessionDirect: Could not send %s signal", "Setup");
@@ -1325,6 +1348,7 @@ send_setup_for_user (GdmSessionDirect *session)
         const char     *display_name;
         const char     *display_device;
         const char     *display_hostname;
+        const char     *display_x11_authority_file;
         const char     *selected_user;
 
         if (session->priv->display_name != NULL) {
@@ -1342,6 +1366,11 @@ send_setup_for_user (GdmSessionDirect *session)
         } else {
                 display_device = "";
         }
+        if (session->priv->display_x11_authority_file != NULL) {
+                display_x11_authority_file = session->priv->display_x11_authority_file;
+        } else {
+                display_x11_authority_file = "";
+        }
         if (session->priv->selected_user != NULL) {
                 selected_user = session->priv->selected_user;
         } else {
@@ -1359,6 +1388,7 @@ send_setup_for_user (GdmSessionDirect *session)
         dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &display_name);
         dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &display_device);
         dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &display_hostname);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &display_x11_authority_file);
         dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &selected_user);
 
         if (! send_dbus_message (session->priv->worker_connection, message)) {
@@ -1389,7 +1419,7 @@ gdm_session_direct_setup_for_user (GdmSession  *session,
         g_return_if_fail (dbus_connection_get_is_connected (impl->priv->worker_connection));
         g_return_if_fail (username != NULL);
 
-        impl->priv->selected_user = g_strdup (username);
+        gdm_session_direct_select_user (session, username);
 
         send_setup_for_user (impl);
 }
@@ -1620,6 +1650,8 @@ open_ck_session (GdmSessionDirect *session)
                 display_device = "";
         }
 
+        g_assert (session->priv->selected_user != NULL);
+
         pwent = getpwnam (session->priv->selected_user);
         if (pwent == NULL) {
                 return FALSE;
@@ -1837,13 +1869,11 @@ gdm_session_direct_select_language (GdmSession *session,
 }
 
 static void
-gdm_session_direct_select_user (GdmSession *session,
-                                const char *text)
+_gdm_session_direct_set_service_name (GdmSessionDirect *session,
+                                      const char       *name)
 {
-        GdmSessionDirect *impl = GDM_SESSION_DIRECT (session);
-
-        g_free (impl->priv->selected_user);
-        impl->priv->selected_user = g_strdup (text);
+        g_free (session->priv->service_name);
+        session->priv->service_name = g_strdup (name);
 }
 
 /* At some point we may want to read these right from
@@ -1881,6 +1911,14 @@ _gdm_session_direct_set_user_x11_authority_file (GdmSessionDirect *session,
         session->priv->user_x11_authority_file = g_strdup (name);
 }
 
+static void
+_gdm_session_direct_set_display_x11_authority_file (GdmSessionDirect *session,
+                                                    const char       *name)
+{
+        g_free (session->priv->display_x11_authority_file);
+        session->priv->display_x11_authority_file = g_strdup (name);
+}
+
 static void
 _gdm_session_direct_set_display_is_local (GdmSessionDirect *session,
                                           gboolean          is)
@@ -1899,6 +1937,9 @@ gdm_session_direct_set_property (GObject      *object,
         self = GDM_SESSION_DIRECT (object);
 
         switch (prop_id) {
+        case PROP_SERVICE_NAME:
+                _gdm_session_direct_set_service_name (self, g_value_get_string (value));
+                break;
         case PROP_DISPLAY_NAME:
                 _gdm_session_direct_set_display_name (self, g_value_get_string (value));
                 break;
@@ -1911,6 +1952,9 @@ gdm_session_direct_set_property (GObject      *object,
         case PROP_USER_X11_AUTHORITY_FILE:
                 _gdm_session_direct_set_user_x11_authority_file (self, g_value_get_string (value));
                 break;
+        case PROP_DISPLAY_X11_AUTHORITY_FILE:
+                _gdm_session_direct_set_display_x11_authority_file (self, g_value_get_string (value));
+                break;
         case PROP_DISPLAY_IS_LOCAL:
                 _gdm_session_direct_set_display_is_local (self, g_value_get_boolean (value));
                 break;
@@ -1931,6 +1975,9 @@ gdm_session_direct_get_property (GObject    *object,
         self = GDM_SESSION_DIRECT (object);
 
         switch (prop_id) {
+        case PROP_SERVICE_NAME:
+                g_value_set_string (value, self->priv->service_name);
+                break;
         case PROP_DISPLAY_NAME:
                 g_value_set_string (value, self->priv->display_name);
                 break;
@@ -1943,6 +1990,9 @@ gdm_session_direct_get_property (GObject    *object,
         case PROP_USER_X11_AUTHORITY_FILE:
                 g_value_set_string (value, self->priv->user_x11_authority_file);
                 break;
+        case PROP_DISPLAY_X11_AUTHORITY_FILE:
+                g_value_set_string (value, self->priv->display_x11_authority_file);
+                break;
         case PROP_DISPLAY_IS_LOCAL:
                 g_value_set_boolean (value, self->priv->display_is_local);
                 break;
@@ -1975,6 +2025,9 @@ gdm_session_direct_dispose (GObject *object)
         g_free (session->priv->display_device);
         session->priv->display_device = NULL;
 
+        g_free (session->priv->display_x11_authority_file);
+        session->priv->display_x11_authority_file = NULL;
+
         g_free (session->priv->server_address);
         session->priv->server_address = NULL;
 
@@ -2041,6 +2094,13 @@ gdm_session_direct_class_init (GdmSessionDirectClass *session_class)
 
         g_type_class_add_private (session_class, sizeof (GdmSessionDirectPrivate));
 
+        g_object_class_install_property (object_class,
+                                         PROP_SERVICE_NAME,
+                                         g_param_spec_string ("service-name",
+                                                              "service name",
+                                                              "service name",
+                                                              "gdm",
+                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
         g_object_class_install_property (object_class,
                                          PROP_DISPLAY_NAME,
                                          g_param_spec_string ("display-name",
@@ -2062,6 +2122,13 @@ gdm_session_direct_class_init (GdmSessionDirectClass *session_class)
                                                                "display is local",
                                                                TRUE,
                                                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+        g_object_class_install_property (object_class,
+                                         PROP_DISPLAY_X11_AUTHORITY_FILE,
+                                         g_param_spec_string ("display-x11-authority-file",
+                                                              "display x11 authority file",
+                                                              "display x11 authority file",
+                                                              NULL,
+                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
         /* not construct only */
         g_object_class_install_property (object_class,
                                          PROP_USER_X11_AUTHORITY_FILE,
@@ -2085,6 +2152,7 @@ GdmSessionDirect *
 gdm_session_direct_new (const char *display_name,
                         const char *display_hostname,
                         const char *display_device,
+                        const char *display_x11_authority_file,
                         gboolean    display_is_local)
 {
         GdmSessionDirect *session;
@@ -2093,6 +2161,7 @@ gdm_session_direct_new (const char *display_name,
                                 "display-name", display_name,
                                 "display-hostname", display_hostname,
                                 "display-device", display_device,
+                                "display-x11-authority-file", display_x11_authority_file,
                                 "display-is-local", display_is_local,
                                 NULL);
 
